
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: HIGHLIGHT_FONT_COLOR, ToggleCharacter, XP

local BreakUpLargeNumbers = BreakUpLargeNumbers
local floor = floor
local GetXPExhaustion = GetXPExhaustion
local IsXPUserDisabled = IsXPUserDisabled
local LEVEL_GAINED = LEVEL_GAINED
local UnitXP = UnitXP
local UnitXPMax = UnitXPMax

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display XP")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI XP", {
	description = "Shows while you're below max level.",
	type = "progress bar",
	label = XP,
	level = UNKNOWN,
	text = UNKNOWN,
	value = "0",
	projectedValue = "0",
	suffix = "%",
	visible = false,
})

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:RegisterEvent("DISABLE_XP_GAIN", "Update")
	self:RegisterEvent("ENABLE_XP_GAIN", "Update")
	self:RegisterEvent("PLAYER_LEVEL_CHANGED", "Update")
	self:RegisterEvent("PLAYER_UPDATE_RESTING", "Update")
	self:RegisterEvent("PLAYER_XP_UPDATE", "Update")
	self:RegisterEvent("UPDATE_EXHAUSTION", "Update")
	self:RegisterEvent("UPDATE_EXPANSION_LEVEL", "Update")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE", "OnConfigUpdate")
	self:Update()
end

function M:OnConfigUpdate(event, key, ...)
	if key == "color.resource.xp" or key == "color.resource.restedXP" then
		self:Update()
	end
end

function M:ShouldBeVisible()
	return C.PLAYER_LEVEL < C.PLAYER_MAX_LEVEL and (not IsXPUserDisabled or not IsXPUserDisabled())
end

function M:Update()
	local currXP = UnitXP("player")
	local nextXP = UnitXPMax("player")
	local percent = (currXP / nextXP) * 100
	local exhaustionThreshold = GetXPExhaustion()
	local color = exhaustionThreshold and C.RESTED_XP_RESOURCE_COLOR or C.XP_RESOURCE_COLOR

	O.barR, O.barG, O.barB = color.r, color.g, color.b
	O.level = LEVEL_GAINED:format(C.PLAYER_LEVEL)
	O.text = C.XP_TEXT:format(BreakUpLargeNumbers(currXP), BreakUpLargeNumbers(nextXP), floor(percent))
	O.value = percent
	O.projectedValue = exhaustionThreshold and (exhaustionThreshold / nextXP) * 100 or 0
	O.visible = self:ShouldBeVisible()
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	ToggleCharacter("PaperDollFrame")
end

function O.OnTooltipShow(tooltip)
	tooltip:SetText(O.label)
	tooltip:AddLine(O.text, HIGHLIGHT_FONT_COLOR:GetRGB())
	tooltip:AddLine(O.level)
end
